@@ -738,7 +738,7 @@ class SessionOptions(Gtk.Window):
     def get_widget_options(self, fn):
         return getattr(self, "%s_options" % fn)
 
-    def bool_cb(self, text, option_name, tooltip_text="", link="") -> Gtk.Switch:
+    def bool_cb(self, text: str, option_name: str, tooltip_text="", link="") -> Gtk.Switch:
         self.attach_label(text, tooltip_text, link)
         fn = option_name.replace("-", "_")
         value = getattr(self.options, fn)
@@ -791,17 +791,17 @@ class SessionOptions(Gtk.Window):
         self.row.increase()
         return btns
 
-    def combo(self, text: str, option_name: str, options, link="") -> Gtk.ComboBoxText:
+    def combo(self, text: str, option_name: str, options: dict, link="") -> Gtk.ComboBoxText:
         self.attach_label(text, "", link)
         fn = option_name.replace("-", "_")
         value = getattr(self.options, fn)
         c = Gtk.ComboBoxText()
-        index = None
+        index = -1
         for i, (v, vlabel) in enumerate(options.items()):
             c.append_text(str(vlabel))
-            if index is None or v == value:
+            if index < 0 or v == value:
                 index = i
-        if index is not None:
+        if index >= 0:
             c.set_active(index)
         al = Gtk.Alignment(xalign=0, yalign=0.5, xscale=0.0, yscale=1.0)
         al.add(c)
@@ -1015,7 +1015,7 @@ class DisplayWindow(SessionOptions):
             for size in SCREEN_SIZES:
                 try:
                     w, h = size.split("x")
-                    size_options[f"{w}x{h}"] =f"{w} x {h}"
+                    size_options[f"{w}x{h}"] = f"{w} x {h}"
                 except (TypeError, ValueError, IndexError):
                     size_options[size] = size
             self.combo("Screen Size", "resize-display", size_options)
@@ -1097,7 +1097,14 @@ class KeyboardWindow(SessionOptions):
         layouts = {
             "": "auto",
         }
-        layouts.update(kbd.get_all_x11_layouts())
+        x11_layouts = kbd.get_all_x11_layouts()
+        # many layouts can have the same name (ie: "English"),
+        # only keep the first one that provides it:
+        names = set()
+        for x11_layout, name in x11_layouts.items():
+            if x11_layout and name not in names:
+                layouts[x11_layout] = name
+                names.add(name)
         self.combo("Keyboard Layout", "keyboard-layout", layouts)
         self.bool_cb("State Synchronization", "keyboard-sync")
         self.bool_cb("Raw Mode", "keyboard-raw")
